@@ -18,7 +18,6 @@
 
 #![cfg_attr(feature="nightly", feature(
     const_fn, // for GILProtected::new (#24111)
-    shared, // for std::ptr::Shared (#27730)
     specialization, // for impl FromPyObject<'source> for Vec<...> (#31844)
 ))]
 
@@ -47,6 +46,17 @@
 //! You obtain a `Python` instance by acquiring the GIL,
 //! and have to pass it into all operations that call into the Python runtime.
 //!
+//! # Python 2.7
+//! The library will use the python3 bindings by default. To use the python2 bindings
+//! you must specific the `python27` feature explicitly in your `Cargo.toml`.
+//!
+//! ```ignore
+//! [dependencies.cpython]
+//! version = "*"
+//! default-features = false
+//! features = ["python27-sys"]
+//! ```
+//! 
 //! # Error Handling
 //! The vast majority of operations in this library will return `PyResult<...>`.
 //! This is an alias for the type `Result<..., PyErr>`.
