@@ -16,15 +16,16 @@
 // OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-extern crate num;
+extern crate num_traits;
 
+use self::num_traits::cast::cast;
 use libc::{c_long, c_double};
 use python::{Python, PythonObject, PyClone};
 use err::{self, PyResult, PyErr};
 use super::object::PyObject;
 use super::exc;
 use ffi;
-use conversion::{ToPyObject, ExtractPyObject};
+use conversion::{ToPyObject, FromPyObject};
 
 /// Represents a Python `int` object.
 ///
@@ -64,7 +65,11 @@ pyobject_newtype!(PyFloat, PyFloat_Check, PyFloat_Type);
 
 #[cfg(feature="python27-sys")]
 impl PyInt {
-    /// Creates a new Python `int` object.
+    /// Creates a new Python 2.7 `int` object.
+    ///
+    /// Note: you might want to call `val.to_py_object(py)` instead
+    /// to avoid truncation if the value does not fit into a `c_long`,
+    /// and to make your code compatible with Python 3.x.
     pub fn new(py: Python, val: c_long) -> PyInt {
         unsafe {
             err::cast_from_owned_ptr_or_panic(py, ffi::PyInt_FromLong(val))
@@ -72,6 +77,11 @@ impl PyInt {
     }
 
     /// Gets the value of this integer.
+    ///
+    /// Warning: `PyInt::value()` is only supported for Python 2.7 `int` objects,
+    /// but not for `long` objects.
+    /// In almost all cases, you can avoid the distinction between these types
+    /// by simply calling `obj.extract::<i32>(py)`.
     pub fn value(&self, _py: Python) -> c_long {
         unsafe { ffi::PyInt_AS_LONG(self.0.as_ptr()) }
     }
@@ -123,7 +133,7 @@ macro_rules! int_fits_c_long(
             if val == -1 && PyErr::occurred(py) {
                 return Err(PyErr::fetch(py));
             }
-            match num::traits::cast::<c_long, $rust_type>(val) {
+            match cast::<c_long, $rust_type>(val) {
                 Some(v) => Ok(v),
                 None => Err(overflow_error(py))
             }
@@ -145,7 +155,7 @@ macro_rules! int_fits_larger_int(
 
         extract!(obj to $rust_type; py => {
             let val = try!(obj.extract::<$larger_type>(py));
-            match num::traits::cast::<$larger_type, $rust_type>(val) {
+            match cast::<$larger_type, $rust_type>(val) {
                 Some(v) => Ok(v),
                 None => Err(overflow_error(py))
             }
@@ -176,7 +186,7 @@ macro_rules! int_convert_u64_or_i64 (
             #[cfg(feature="python27-sys")]
             fn to_py_object(&self, py: Python) -> PyObject {
                 unsafe {
-                    let ptr = match num::traits::cast::<$rust_type, c_long>(*self) {
+                    let ptr = match cast::<$rust_type, c_long>(*self) {
                         Some(v) => ffi::PyInt_FromLong(v),
                         None => $pylong_from_ll_or_ull(*self)
                     };
@@ -192,23 +202,16 @@ macro_rules! int_convert_u64_or_i64 (
             }
         }
 
-        impl <'prepared> ExtractPyObject<'prepared> for $rust_type {
-            type Prepared = PyObject;
-
-            #[inline]
-            fn prepare_extract(py: Python, obj: &PyObject) -> PyResult<Self::Prepared> {
-                Ok(obj.clone_ref(py))
-            }
-
+        impl <'source> FromPyObject<'source> for $rust_type {
             #[cfg(feature="python27-sys")]
-            fn extract(py: Python, obj: &'prepared PyObject) -> PyResult<$rust_type> {
+            fn extract(py: Python, obj: &'source PyObject) -> PyResult<$rust_type> {
                 let ptr = obj.as_ptr();
 
                 unsafe {
                     if ffi::PyLong_Check(ptr) != 0 {
                         err_if_invalid_value(py, !0, $pylong_as_ull_or_ull(ptr))
                     } else if ffi::PyInt_Check(ptr) != 0 {
-                        match num::traits::cast::<c_long, $rust_type>(ffi::PyInt_AS_LONG(ptr)) {
+                        match cast::<c_long, $rust_type>(ffi::PyInt_AS_LONG(ptr)) {
                             Some(v) => Ok(v),
                             None => Err(overflow_error(py))
                         }
@@ -220,7 +223,7 @@ macro_rules! int_convert_u64_or_i64 (
             }
 
             #[cfg(feature="python3-sys")]
-            fn extract(py: Python, obj: &'prepared PyObject) -> PyResult<$rust_type> {
+            fn extract(py: Python, obj: &'source PyObject) -> PyResult<$rust_type> {
                 let ptr = obj.as_ptr();
                 unsafe {
                     if ffi::PyLong_Check(ptr) != 0 {
