@@ -23,8 +23,9 @@ Additionally, it generates a Rust struct of the same name, which allows accessin
 instances of that Python class from Rust.
 
 # Syntax
-`py_class!(class MyType |py| { ... })`
+`py_class!(pub class MyType |py| { ... })`
 
+* `pub` makes the generated Rust struct visible outside the current module. It has no effect on the visibility from Python.
 * `MyType` is the name of the Python class.
 * `py` is an identifier that will be made available as a variable of type `Python`
 in all function bodies.
@@ -206,6 +207,7 @@ py_class!(class ClassWithGCSupport |py| {
 ```
 
 Caution: `__traverse__` may be called by the garbage collector:
+
   * during any python operation that takes a `Python` token as argument
   * indirectly from the `PyObject` (or derived type) `Drop` implementation
   * if your code releases the GIL, at any time by other threads.
@@ -278,12 +280,15 @@ py_class!(class MyIterator |py| {
 
 ## Comparison operators
 
-  * `def __richcmp__(&self, other: impl ToPyObject, op: CompareOp) -> PyResult<impl ToPyObject>`
+  * `def __richcmp__(&self, other: impl FromPyObject, op: CompareOp) -> PyResult<impl ToPyObject>`
 
-    Overloads Python comparison operations (`==`, `!=`, `<`, `<=`, `>`, and `>=`). The `op`
-    argument indicates the comparison operation being performed.
+    Overloads Python comparison operations (`==`, `!=`, `<`, `<=`, `>`, and `>=`).
+    The `op` argument indicates the comparison operation being performed.
     The return type will normally be `PyResult<bool>`, but any Python object can be returned.
 
+    If `other` is not of the type specified in the signature, the generated code will
+    automatically `return NotImplemented`.
+
   * `def __hash__(&self) -> PyResult<impl PrimInt>`
 
     Objects that compare equal must have the same hash value.
@@ -401,6 +406,36 @@ macro_rules! py_class {
             /* info: */ {
                 /* base_type: */ $crate::PyObject,
                 /* size: */ <$crate::PyObject as $crate::py_class::BaseObject>::size(),
+                /* class_visibility: */ {},
+                /* gc: */ {
+                    /* traverse_proc: */ None,
+                    /* traverse_data: */ [ /*name*/ ]
+                },
+                /* data: */ [ /* { offset, name, type } */ ]
+                // TODO: base type, documentation, ...
+            }
+            /* slots: */ {
+                /* type_slots */  [ /* slot: expr, */ ]
+                /* as_number */   [ /* slot: expr, */ ]
+                /* as_sequence */ [ /* slot: expr, */ ]
+                /* as_mapping */  [ /* slot: expr, */ ]
+                /* setitem_delitem */ [
+                    sdi_setitem: {},
+                    sdi_delitem: {},
+                ]
+            }
+            /* impls: */ { /* impl body */ }
+            /* members: */ { /* ident = expr; */ }
+        }
+    );
+    (pub class $class:ident |$py: ident| { $( $body:tt )* }) => (
+        py_class_impl! {
+            { $( $body )* }
+            $class $py
+            /* info: */ {
+                /* base_type: */ $crate::PyObject,
+                /* size: */ <$crate::PyObject as $crate::py_class::BaseObject>::size(),
+                /* class_visibility: */ {pub},
                 /* gc: */ {
                     /* traverse_proc: */ None,
                     /* traverse_data: */ [ /*name*/ ]
@@ -436,4 +471,3 @@ macro_rules! py_class_impl_item {
         }
     }}
 }
-
