@@ -43,12 +43,15 @@ base_case = '''
         /* info: */ {
             $base_type:ty,
             $size:expr,
+            { $( $class_visibility:tt )* },
             $gc:tt,
             /* data: */ [ $( { $data_offset:expr, $data_name:ident, $data_ty:ty } )* ]
         }
         $slots:tt { $( $imp:item )* } $members:tt
     } => {
-        struct $class { _unsafe_inner: $crate::PyObject }
+        py_coerce_item! {
+            $($class_visibility)* struct $class { _unsafe_inner: $crate::PyObject }
+        }
 
         py_impl_to_py_object_for_python_object!($class);
         py_impl_from_py_object_for_python_object!($class);
@@ -149,7 +152,7 @@ base_case = '''
                                     $crate::PyType::from_type_ptr(py, &mut TYPE_OBJECT)
                                 } else {
                                     // automatically initialize the class on-demand
-                                    <$class as $crate::py_class::PythonObjectFromPyClassMacro>::initialize(py)
+                                    <$class as $crate::py_class::PythonObjectFromPyClassMacro>::initialize(py, None)
                                         .expect(concat!("An error occurred while initializing class ", stringify!($class)))
                                 }
                             }
@@ -157,7 +160,7 @@ base_case = '''
                     }
 
                     impl $crate::py_class::PythonObjectFromPyClassMacro for $class {
-                        fn initialize(py: $crate::Python) -> $crate::PyResult<$crate::PyType> {
+                        fn initialize(py: $crate::Python, module_name: Option<&str>) -> $crate::PyResult<$crate::PyType> {
                             unsafe {
                                 if $crate::py_class::is_ready(py, &TYPE_OBJECT) {
                                     return Ok($crate::PyType::from_type_ptr(py, &mut TYPE_OBJECT));
@@ -166,15 +169,20 @@ base_case = '''
                                     concat!("Reentrancy detected: already initializing class ",
                                     stringify!($class)));
                                 INIT_ACTIVE = true;
-                                let res = init(py);
+                                let res = init(py, module_name);
                                 INIT_ACTIVE = false;
                                 res
                             }
                         }
+
+                        fn add_to_module(py: $crate::Python, module: &$crate::PyModule) -> $crate::PyResult<()> {
+                            let ty = <$class as $crate::py_class::PythonObjectFromPyClassMacro>::initialize(py, module.name(py).ok())?;
+                            module.add(py, stringify!($class), ty)
+                        }
                     }
 
-                    fn init($py: $crate::Python) -> $crate::PyResult<$crate::PyType> {
-                        py_class_type_object_dynamic_init!($class, $py, TYPE_OBJECT, $slots);
+                    fn init($py: $crate::Python, module_name: Option<&str>) -> $crate::PyResult<$crate::PyType> {
+                        py_class_type_object_dynamic_init!($class, $py, TYPE_OBJECT, module_name, $slots);
                         py_class_init_members!($class, $py, TYPE_OBJECT, $members);
                         unsafe {
                             if $crate::_detail::ffi::PyType_Ready(&mut TYPE_OBJECT) == 0 {
@@ -243,6 +251,7 @@ def generate_case(pattern, old_info=None, new_info=None, new_impl=None, new_slot
         write('\n/* info: */ {\n')
         write('$base_type: ty,\n')
         write('$size: expr,\n')
+        write('$class_visibility: tt,\n')
         write('$gc: tt,\n')
         write('[ $( $data:tt )* ]\n')
         write('}\n')
@@ -326,6 +335,7 @@ def data_decl():
         /* info: */ {
             $base_type,
             /* size: */ $crate::py_class::data_new_size::<$data_type>($size),
+            $class_visibility,
             $gc,
             /* data: */ [
                 $($data)*
@@ -387,6 +397,7 @@ def traverse_and_clear():
         /* info: */ {
             $base_type: ty,
             $size: expr,
+            $class_visibility: tt,
             /* gc: */ {
                 /* traverse_proc: */ None,
                 $traverse_data: tt
@@ -398,6 +409,7 @@ def traverse_and_clear():
         /* info: */ {
             $base_type,
             $size,
+            $class_visibility,
             /* gc: */ {
                 /* traverse_proc: */ $class::__traverse__,
                 $traverse_data
